/*
 * xvdat.c - load and save routines for datfile format pictures & sequences
 *
 * LoadDAT(fname, pinfo)  -  loads a datfile
 * WriteDAT(fp,pic,ptype,w,h,comment)  - saves a datfile
 *
 * Andrew Huang       bunnie@mit.edu      September 1, 1996
 */

#include "copyright.h"

#include "xv.h"
#include "local/dat.h"

static int garbage;
static long numgot, filesize;

static int datError PARM((char *, char *));

static char *bname;

/*******************************************/
int LoadDAT(fname, pinfo)
     char    *fname;
     PICINFO *pinfo;
/*******************************************/
{
  /* returns '1' on success */

  DATFILE  *fp;
  int    c, c1;
  int    maxv, rv;
  char   *datname;
  long   i;
  long   nsets, nchan, dim[10];
  int    maxdim = 10;
  char   *buf; 
  char   *tmpbuf;
  long   x, y, width, height;

  garbage = maxv = rv = 0;
  bname = BaseName(fname);

  /* compute name of datfile directory proper */
  i = strlen( fname );
  i -= strlen( "/descriptor" );    /* subtract the length of descriptor */
  i++;  /* add on an extra byte for the terminating /0 */
  datname = malloc( sizeof(char) * i );  /* not checking malloc return...bad */
  strncpy( datname, fname, i - 1 );  /* copy over everything but the bname */
  datname[i - 1] = '\0';             /* append a null character */

  pinfo->pic     = (byte *) NULL;
  pinfo->comment = (char *) NULL;

  /* open the file */
  fp = d_open(datname,"r");
  if (!fp) return (datError(bname, "can't open file"));

  /* read in header information */
  if( 0 == d_getdim(fp, &nsets, &nchan, dim, maxdim) )
    return( datError( bname, "can't dimension file"));

/*  printf("nsets: %ld, nchan: %ld, dim[0]: %ld, dim[1]: %ld\n", nsets, nchan, dim[0], dim[1] ); */

  if( nsets > 1 )
    return( datError( bname, "can't open sequences yet"));

  pinfo->w = dim[1];  pinfo->h = dim[0];
  width = dim[1];  height = dim[0];
  pinfo->normw = pinfo->w;   pinfo->normh = pinfo->h;

  /* note:  pic, type, r,g,b, frmInfo, shortFrm, and colorType fields of
     picinfo struct are filled in in the format-specific loaders */
  
  if( nchan == 3 ) {
    pinfo->type = PIC24;
    pinfo->colType = F_FULLCOLOR;
    buf = calloc( (size_t) dim[1] * dim[0] * 3, (size_t) 1 );
    tmpbuf = calloc( (size_t) dim[1] * dim[0] * 3, (size_t) 1 );
    d_readc( fp, buf, (int) dim[1] * dim[0] * 3, (int) 'b' );

    /*** perform format conversion ***/
    /* datfiles are in R, G, B block channels, ie, a 2D array of reds then */
    /* greens then blues.  XV takes a 2D array of RGB tuples, where */
    /* consecutive bytes are R, G, B values. */

    /* i corresponds to datfile index */
    /* x, y correspond to the xv buffer coordinates */

    /* xform red first */
    for( i = 0, y = 0; y < height; y++ ) { 
      for( x = 0; x < width; x++, i++ ) {
	tmpbuf[x * 3 + y * 3 * width] = buf[i];
      } /* for */
    } /* for */
    /* xform green next */
    for( y = 0; y < height; y++ ) { 
      for( x = 0; x < width; x++, i++ ) {
	tmpbuf[x * 3 + y * 3 * width + 1] = buf[i];
      } /* for */
    } /* for */
    /* xform blue last */
    for( y = 0; y < height; y++ ) { 
      for( x = 0; x < width; x++, i++ ) {
	tmpbuf[x * 3 + y * 3 * width + 2] = buf[i];
      } /* for */
    } /* for */

    WaitCursor();
    sprintf(pinfo->fullInfo, "datfile format.  (%ld bytes)", dim[1]*dim[0]*3);
    sprintf(pinfo->shrtInfo, "%dx%d datfile.", dim[1], dim[0]);

    pinfo->pic = tmpbuf;
    free( buf ); /* clean up */

  } /* if */
  else if( nchan == 1 ) {
    pinfo->type = PIC8;
    pinfo->colType = F_GREYSCALE;
    d_readcm( fp, &buf, dim[1] * dim[0], (int) 'b' );
    WaitCursor();
    sprintf(pinfo->fullInfo, "datfile format.  (%ld bytes)", dim[1]*dim[0] );
    sprintf(pinfo->shrtInfo, "%dx%d datfile.", dim[1], dim[0]);

    pinfo->pic = buf;

    /* fill in a greyscale colormap where maxv maps to 255 */
    for (i=0; i<=255; i++)
      pinfo->r[i] = pinfo->g[i] = pinfo->b[i] = (i*255)/255;

  } /* else if */
  else
    return( datError( bname, "can only handle 1 or 3 channel datfiles\n" ));
  
  pinfo->frmType = F_DAT;  /* uncomment when save type is taken care of */ 
/*  pinfo->frmType = 0;*/
  pinfo->comment = (char *) NULL;

	   
  d_close(fp);

  rv = 1;

  if (!rv) {
    if (pinfo->pic) free(pinfo->pic);
    if (pinfo->comment) free(pinfo->comment);
    pinfo->pic     = (byte *) NULL;
    pinfo->comment = (char *) NULL;
  } /* if */

  /* free up temp memory */
  free( datname );

/*  printf("\nmax;%d tif:%d jpg:%d dat:%d\n", F_MAXFMTS, F_TIFINC, F_JPGINC, F_DATINC); */
/*  printf("it ended.\n"); */
  return rv;
}  


/*******************************************/
int WriteDAT(fp,pic,ptype,w,h,comment)
     DATFILE *fp;
     byte *pic;
     int   ptype, w,h;
     char *comment;
{
  /* locals */
  long dim[2];
  char *buf;
  long i;
  long x, y;

  /* body */
  d_settype(fp, (int) 'b');   /* all outputs are in byte unsigned format */
  dim[0] = (long) h;
  dim[1] = (long) w;
  if( ptype == PIC24 ) 
    d_setdim(fp, 1L, 3L, dim, 2);  /* one set three channels, ndim = 2 */
  else if( ptype == PIC8 )
    d_setdim(fp, 1L, 1L, dim, 2);  /* one set one channel, ndim = 2 */
  else
    return( -1 );

  d_setkey(fp, COMMENT, "File created by xv-3.10a with datfile extensions added by Andrew Huang (bunnie@mit.edu)." );
  if( comment != NULL )
    d_setkey(fp, COMMENT, comment );

  if( ptype == PIC24 ) { /* gotta deal with color images...three channels! */
    buf = calloc( (size_t) w * h * 3, (size_t) 1 );
    
    /** transform from XV RGB tuples to datfile 2D color blocks (channels) **/
    /* i corresponds to datfile index */
    /* x, y correspond to the xv buffer coordinates */
    
    /* xform red first */
    for( i = 0, y = 0; y < h; y++ ) { 
      for( x = 0; x < w; x++, i++ ) {
	buf[i] = (char) pic[x * 3 + y * 3 * w];
      } /* for */
    } /* for */
    /* xform green next */
    for( y = 0; y < h; y++ ) { 
      for( x = 0; x < w; x++, i++ ) {
	buf[i] = (char) pic[x * 3 + y * 3 * w + 1];
      } /* for */
    } /* for */
    /* xform blue last */
    for( y = 0; y < h; y++ ) { 
      for( x = 0; x < w; x++, i++ ) {
	buf[i] = (char) pic[x * 3 + y * 3 * w + 2];
      } /* for */
    } /* for */
    
    d_writec(fp, buf, w * h * 3, (int) 'b' );
    
  } /* if */
  else { /* life is easy -- ahhh, grayscale */
    d_writec(fp, pic, w * h, (int) 'b' );
  } /* else */
  
  d_flush(fp);

  return(0);
}

/*******************************************/
static int datError(fname, st)
     char *fname, *st;
{
  SetISTR(ISTR_WARNING,"%s:  %s", fname, st);
  return 0;
}

